/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package sig.controller.geocoder;

import com.opencsv.CSVReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.io.StringReader;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import org.json.JSONException;
import org.json.JSONObject;
import org.postgresql.copy.CopyManager;
import org.postgresql.core.BaseConnection;
import org.primefaces.event.FileUploadEvent;
import org.primefaces.model.UploadedFile;
import sig.controller.application.LoginMB;
import sig.model.jdbc.ConnectionJdbcMB;

/**
 *
 * @author nelson
 */
@ManagedBean
@ViewScoped
public class CsvDataMB implements Serializable {

    private double resultLongitude;
    private double resultLatitude;
    /**
     * CSV proccess variables
     */
    private List<String> attributeNames; //Nombres de los atributos
    private String clusteringAttributeValue; //Valor atributo de agrupamiento 
    private String addressAttribute; //Seleccion del atributo que contiene direcciones urbanas
    private String neighborhoodAttribute; //Seleccion del atributo que contiene el barrio
    private String lonAttribute; //Seleccion del atributo que contiene la longitud del punto en EPSG:3857
    private String latAttribute; //Seleccion del atributo que contiene la latitud del punto en EPSG:3857

    private String clusteringValues = "undefined";
    private boolean clusteringData = false; //Los datos serán agrupados o no
    private boolean blockAddressesType = false; //Por defecto el tipo de direccion seleccionada es Malla vial

    private String csvFileName = "Ninguno"; //nombre del archivo
    private byte[] csvFileData; //datos del archivo
    private int tuples; //cantidad total de registros
    private int processedTuples; //cantidad de registros geocodificados

    private StringBuilder sb;
    private CopyManager cpManager;
    private String sql;

    private ConnectionJdbcMB connectionJdbcMB;
    private LoginMB loginMB;
    private String sessionId;

    /**
     * @Resource(name = "jdbc/pastoviu") private DataSource ds;
     */
    /**
     * Creates a new instance of addressMB
     */
    public CsvDataMB() {
    }

    @PostConstruct
    public void init() {
        FacesContext fCtx = FacesContext.getCurrentInstance();
        connectionJdbcMB = (ConnectionJdbcMB) fCtx.getApplication().evaluateExpressionGet(fCtx, "#{connectionJdbcMB}", ConnectionJdbcMB.class);
        loginMB = (LoginMB) fCtx.getApplication().evaluateExpressionGet(fCtx, "#{loginMB}", LoginMB.class);
        sessionId = loginMB.getCurrentUser().getUserId().toString();
        System.out.println("PASTO VIEW info: Ejecución método PostConstruct. Id Sesion: " + sessionId);
    }

    /**
     * This method is responsible to obtain the names of CSV file attributes
     *
     * @param event
     */
    public void loadCSVFile(FileUploadEvent event) {
        try {

            UploadedFile csvFile = event.getFile();
            csvFileData = csvFile.getContents();
            csvFileName = csvFile.getFileName();
            CSVReader reader = new CSVReader(new InputStreamReader(new ByteArrayInputStream(csvFileData)));
            attributeNames = Arrays.asList(reader.readNext());

        } catch (IOException ex) {
            System.out.println("-- PASTO VIEW info: ERROR 2 - Clase " + this.getClass().getName() + ". " + ex.getMessage() + " --");
        }
    }

    /**
     * This method is responsible to read the CSV file data and insert them in
     * the database.
     *
     * @
     */
    public void processCSV() {
        try {

            clusteringValues = "undefined";
            sql = ""
                    + "DELETE FROM csv_data\n"
                    + "WHERE session_id LIKE '" + sessionId + "';";
            connectionJdbcMB.non_query(sql);

            //INSERCION DE DATOS CON COPYMANAGER
            sb = new StringBuilder();
            cpManager = new CopyManager((BaseConnection) connectionJdbcMB.getConn());
            CSVReader reader = new CSVReader(new InputStreamReader(new ByteArrayInputStream(csvFileData)));
            reader.readNext(); //Se descarta la linea con los nombres de los atributos
            String[] nextLine;

            //Obtencion timestamp registro de los datos
            Date date = new Date();
            long time = date.getTime();
            Timestamp ts = new Timestamp(time);

            tuples = 0; //total registros
            processedTuples = 0; //total registros geocodificados

            while ((nextLine = reader.readNext()) != null) {
                tuples++;
                JSONObject properties = new JSONObject();
                for (int i = 0; i < nextLine.length; i++) {
                    properties.put(attributeNames.get(i), nextLine[i]);
                }
                if (!blockAddressesType) { //malla vial
                    addressGeocoding(properties.getString(addressAttribute));
                } else {//manzanas
                    addressGeocoding(properties.getString(neighborhoodAttribute), properties.getString(addressAttribute));
                }
                properties.put("lon", resultLongitude);
                properties.put("lat", resultLatitude);

                if (resultLongitude != 0 && resultLatitude != 0) {
                    processedTuples++;
                }
                if (clusteringData) {
                    sb.
                            append(sessionId).append("\t").
                            append(ts).append("\t").
                            append(properties.toString()).append("\t").
                            append(properties.getString(clusteringAttributeValue)).append("\t").
                            append(properties.getDouble("lon")).append("\t").
                            append(properties.getDouble("lat")).append("\n");

                } else {
                    sb.
                            append(sessionId).append("\t").
                            append(ts).append("\t").
                            append(properties.toString()).append("\t").
                            append("-").append("\t").
                            append(properties.getDouble("lon")).append("\t").
                            append(properties.getDouble("lat")).append("\n");

                }

            }
            cpManager.copyIn("COPY csv_data FROM STDIN", new StringReader(sb.toString()));
            sb.delete(0, sb.length());

            //valores para agrupar capas
            if (clusteringData) {
                clusteringValues = "";
                sql = ""
                        + "SELECT DISTINCT clustering_values AS values \n"
                        + "FROM csv_data\n"
                        + "WHERE session_id LIKE '" + sessionId + "' AND (lon <> 0 AND lat <> 0)\n"
                        + "ORDER BY clustering_values;";
                ResultSet rs = connectionJdbcMB.consult(sql);
                while (rs.next()) {
                    clusteringValues = clusteringValues + rs.getString("values") + ",";
                }
            }
            addFacesSuccessMessage("Procesamiento del archivo '" + csvFileName + "' finalizado. \nTuplas procesadas; " + processedTuples + "/" + tuples);

        } catch (NullPointerException | IOException | SQLException | JSONException ex) {

            if (csvFileData == null) {
                addFacesErrorMessage("No se ha seleccionado un archivo CSV");
            } else if (clusteringData && clusteringAttributeValue.compareTo("none") == 0) {
                FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, "ERROR", "Agrupamiento de datos activado. Por favor seleccione un atributo.");
                FacesContext.getCurrentInstance().addMessage("successInfo", facesMsg);
            } else {
                addFacesErrorMessage("La configuración seleccionada no es valida.");
                System.out.println("-- PASTO VIEW info: ERROR 3 - Clase " + this.getClass().getName() + ". " + ex.getMessage() + " --");
            }
        }
    }

    /**
     * This method defines the query which be send to the address geocoding
     * function (road network type).
     *
     * @param addr
     */
    public void addressGeocoding(String addr) {
        String address = addr.replace("'", "");//algunos datos pueden venir entre comillas simples, estas se eliminan
        //System.out.println("Parametro ingresado: " + address);
        sql = ""
                + "WITH geocoded AS(\n"
                + "	SELECT\n"
                + "		geocode_address('" + address + "') AS result\n"
                + ")\n"
                + "SELECT\n"
                + "	(result).address,\n"
                + "	(result).neighborhood,\n"
                + "	(result).commune,\n"
                + "	(result).lon,\n"
                + "	(result).lat\n"
                + "FROM\n"
                + "	geocoded\n"
                + "WHERE\n"
                + "	(result).lon IS NOT NULL AND (result).lat IS NOT NULL;";
        //System.out.println("query:\n" + sql);
        geocoderQuery(sql);
    }

    /**
     * This method defines the query which be send to the address geocoding
     * function (block houseid type).
     *
     * @param neigh
     * @param addr
     */
    public void addressGeocoding(String neigh, String addr) {
        String address = addr.replace("'", ""); //algunos datos pueden venir entre comillas simples, estas se eliminan
        String neighborhood = neigh.replace("'", "");
        sql = ""
                + "WITH query AS (\n"
                + "	SELECT \n"
                + "		geocode_address('" + neighborhood + "','" + address + "') AS result\n"
                + ")\n"
                + "SELECT\n"
                + "	(result).address,\n"
                + "	(result).neighborhood,\n"
                + "	(result).commune,\n"
                + "	(result).lon,\n"
                + "	(result).lat\n"
                + "FROM\n"
                + "	query\n"
                + "WHERE\n"
                + "	(result).lon IS NOT NULL AND (result).lat IS NOT NULL;";
        geocoderQuery(sql);
    }

    /**
     * This method performs the connection to the geocoder and execute the
     * geocoding query.
     *
     * @param sql
     */
    public void geocoderQuery(String sql) {

        try {
            ResultSet rs = connectionJdbcMB.consult(sql);

            if (rs.next()) {
                resultLongitude = rs.getDouble("lon");
                resultLatitude = rs.getDouble("lat");
            } else {
                resultLongitude = 0;
                resultLatitude = 0;
            }

        } catch (SQLException ex) {
            System.out.println("Error 1 en " + this.getClass().getName() + ":" + ex.toString());
        }
    }

    /**
     * Methods used to show faces messages in the view.
     *
     * @param msg
     */
    public void addFacesErrorMessage(String msg) {
        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, "ERROR", msg);
        FacesContext.getCurrentInstance().addMessage(null, facesMsg);
    }

    public void addFacesSuccessMessage(String msg) {
        FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "INFO", msg);
        FacesContext.getCurrentInstance().addMessage(null, facesMsg);
    }

    /*Getters and Setters*/

    public int getProcessedTuples() {
        return processedTuples;
    }

    public void setProcessedTuples(int processedTuples) {
        this.processedTuples = processedTuples;
    }

    public double getResultLongitude() {
        return resultLongitude;
    }

    public void setResultLongitude(double resultLongitude) {
        this.resultLongitude = resultLongitude;
    }

    public double getResultLatitude() {
        return resultLatitude;
    }

    public void setResultLatitude(double resultLatitude) {
        this.resultLatitude = resultLatitude;
    }

    /*CSV process*/
    public List<String> getAttributeNames() {
        return attributeNames;
    }

    public void setAttributeNames(List<String> attributeNames) {
        this.attributeNames = attributeNames;
    }

    public String getClusteringAttributeValue() {
        return clusteringAttributeValue;
    }

    public void setClusteringAttributeValue(String clusteringAttributeValue) {
        this.clusteringAttributeValue = clusteringAttributeValue;
    }

    public String getAddressAttribute() {
        return addressAttribute;
    }

    public void setAddressAttribute(String addressAttribute) {
        this.addressAttribute = addressAttribute;
    }

    public String getLonAttribute() {
        return lonAttribute;
    }

    public void setLonAttribute(String lonAttribute) {
        this.lonAttribute = lonAttribute;
    }

    public String getLatAttribute() {
        return latAttribute;
    }

    public void setLatAttribute(String latAttribute) {
        this.latAttribute = latAttribute;
    }

    public boolean isClusteringData() {
        return clusteringData;
    }

    public void setClusteringData(boolean clusteringData) {
        this.clusteringData = clusteringData;
    }

    public boolean isBlockAddressesType2() {
        return blockAddressesType;
    }

    public void setBlockAddressesType2(boolean blockAddressesType) {
        this.blockAddressesType = blockAddressesType;
    }

    public String getNeighborhoodAttribute() {
        return neighborhoodAttribute;
    }

    public void setNeighborhoodAttribute(String neighborhoodAttribute) {
        this.neighborhoodAttribute = neighborhoodAttribute;
    }

    public String getCsvFileName() {
        return csvFileName;
    }

    public void setCsvFileName(String csvFileName) {
        this.csvFileName = csvFileName;
    }

    public String getClusteringValues() {
        return clusteringValues;
    }

    public void setClusteringValues(String clusteringValues) {
        this.clusteringValues = clusteringValues;
    }

    public String getSessionId() {
        return sessionId;
    }

    public void setSessionId(String sessionId) {
        this.sessionId = sessionId;
    }

}
